@@ -1,8 +1,8 @@
 
 #include "squid.h"
 #include "base/AsyncJobCalls.h"
+#include "base/TextException.h"
 #include "BodyPipe.h"
-#include "TextException.h"
 
 CBDATA_CLASS_INIT(BodyPipe);
 
@@ -129,7 +129,7 @@ void BodyConsumer::stopConsumingFrom(RefCount<BodyPipe> &pipe)
 BodyPipe::BodyPipe(Producer *aProducer): theBodySize(-1),
         theProducer(aProducer), theConsumer(0),
         thePutSize(0), theGetSize(0),
-        mustAutoConsume(false), abortedConsumption(false), isCheckedOut(false)
+        mustAutoConsume(false), isCheckedOut(false)
 {
     // TODO: teach MemBuf to start with zero minSize
     // TODO: limit maxSize by theBodySize, when known?
@@ -148,6 +148,7 @@ BodyPipe::~BodyPipe()
 void BodyPipe::setBodySize(uint64_t aBodySize)
 {
     assert(!bodySizeKnown());
+    assert(aBodySize >= 0);
     assert(thePutSize <= aBodySize);
 
     // If this assert fails, we need to add code to check for eof and inform
@@ -182,6 +183,15 @@ uint64_t BodyPipe::unproducedSize() const
     return bodySize() - thePutSize; // bodySize() asserts that size is known
 }
 
+void BodyPipe::expectProductionEndAfter(uint64_t size)
+{
+    const uint64_t expectedSize = thePutSize + size;
+    if (bodySizeKnown())
+        Must(bodySize() == expectedSize);
+    else
+        theBodySize = expectedSize;
+}
+
 void
 BodyPipe::clearProducer(bool atEof)
 {
@@ -230,8 +240,6 @@ BodyPipe::setConsumerIfNotLate(const Consumer::Pointer &aConsumer)
         return false;
     }
 
-    Must(!abortedConsumption); // did not promise to never consume
-
     theConsumer = aConsumer;
     debugs(91,7, HERE << "set consumer" << status());
     if (theBuf.hasContent())
@@ -242,30 +250,25 @@ BodyPipe::setConsumerIfNotLate(const Consumer::Pointer &aConsumer)
     return true;
 }
 
+// When BodyPipe consumer is gone, all events for that consumer must not
+// reach the new consumer (if any). Otherwise, the calls may go out of order
+// (if _some_ calls are dropped due to the ultimate destination being
+// temporary NULL). The code keeps track of the number of outstanding
+// events and skips that number if consumer leaves. TODO: when AscyncCall
+// support is improved, should we just schedule calls directly to consumer?
 void
 BodyPipe::clearConsumer()
 {
     if (theConsumer.set()) {
         debugs(91,7, HERE << "clearing consumer" << status());
         theConsumer.clear();
-        // do not abort if we have not consumed so that HTTP or ICAP can retry
-        // benign xaction failures due to persistent connection race conditions
-        if (consumedSize())
-            expectNoConsumption();
-    }
-}
-
-void
-BodyPipe::expectNoConsumption()
-{
-    Must(!theConsumer);
-    if (!abortedConsumption && !exhausted()) {
-        AsyncCall::Pointer call= asyncCall(91, 7,
-                                           "BodyProducer::noteBodyConsumerAborted",
-                                           BodyProducerDialer(theProducer,
-                                                              &BodyProducer::noteBodyConsumerAborted, this));
-        ScheduleCallHere(call);
-        abortedConsumption = true;
+        if (consumedSize() && !exhausted()) {
+            AsyncCall::Pointer call= asyncCall(91, 7,
+                                               "BodyProducer::noteBodyConsumerAborted",
+                                               BodyProducerDialer(theProducer,
+                                                                  &BodyProducer::noteBodyConsumerAborted, this));
+            ScheduleCallHere(call);
+        }
     }
 }
 
@@ -343,7 +346,7 @@ BodyPipe::undoCheckOut(Checkout &checkout)
     // raw buffers should always check them in (possibly unchanged)
     // instead of relying on the automated undo mechanism of Checkout.
     // The code can always use a temporary buffer to accomplish that.
-    Must(checkout.checkedOutSize == currentSize);
+    assert(checkout.checkedOutSize == currentSize);
 }
 
 // TODO: Optimize: inform consumer/producer about more data/space only if
@@ -423,9 +426,9 @@ const char *BodyPipe::status() const
 
     outputBuffer.append(" [", 2);
 
-    outputBuffer.Printf("%" PRIu64 "<=%" PRIu64, theGetSize, thePutSize);
+    outputBuffer.Printf("%"PRIu64"<=%"PRIu64, theGetSize, thePutSize);
     if (theBodySize >= 0)
-        outputBuffer.Printf("<=%" PRId64, theBodySize);
+        outputBuffer.Printf("<=%"PRId64, theBodySize);
     else
         outputBuffer.append("<=?", 3);
 
@@ -439,8 +442,6 @@ const char *BodyPipe::status() const
 
     if (mustAutoConsume)
         outputBuffer.append(" A", 2);
-    if (abortedConsumption)
-        outputBuffer.append(" !C", 3);
     if (isCheckedOut)
         outputBuffer.append(" L", 2); // Locked
 
@@ -462,13 +463,8 @@ BodyPipeCheckout::BodyPipeCheckout(BodyPipe &aPipe): pipe(aPipe),
 
 BodyPipeCheckout::~BodyPipeCheckout()
 {
-    if (!checkedIn) {
-        // Do not pipe.undoCheckOut(*this) because it asserts or throws
-        // TODO: consider implementing the long-term solution discussed at
-        // http://www.mail-archive.com/squid-dev@squid-cache.org/msg07910.html
-        debugs(91,2, HERE << "Warning: cannot undo BodyPipeCheckout");
-        pipe.checkIn(*this);
-    }
+    if (!checkedIn)
+        pipe.undoCheckOut(*this);
 }
 
 void
