@@ -33,19 +33,18 @@
  */
 
 #include "squid.h"
+#include "base/TextException.h"
 #include "Server.h"
 #include "Store.h"
 #include "fde.h" /* for fd_table[fd].closing */
 #include "HttpRequest.h"
 #include "HttpReply.h"
-#include "TextException.h"
 #include "errorpage.h"
 #include "SquidTime.h"
 
 #if USE_ADAPTATION
 #include "adaptation/AccessCheck.h"
 #include "adaptation/Iterator.h"
-#include "base/AsyncCall.h"
 #endif
 
 // implemented in client_side_reply.cc until sides have a common parent
@@ -58,8 +57,6 @@ ServerStateData::ServerStateData(FwdState *theFwdState): AsyncJob("ServerStateDa
         , adaptationAccessCheckPending(false)
         , startedAdaptation(false)
 #endif
-        ,theVirginReply(NULL),
-        theFinalReply(NULL)
 {
     fwd = theFwdState;
     entry = fwd->entry;
@@ -279,8 +276,7 @@ ServerStateData::noteMoreBodyDataAvailable(BodyPipe::Pointer bp)
         return;
     }
 #endif
-    if (requestBodySource == bp)
-        handleMoreRequestBodyAvailable();
+    handleMoreRequestBodyAvailable();
 }
 
 // the entire request or adapted response body was provided, successfully
@@ -293,8 +289,7 @@ ServerStateData::noteBodyProductionEnded(BodyPipe::Pointer bp)
         return;
     }
 #endif
-    if (requestBodySource == bp)
-        handleRequestBodyProductionEnded();
+    handleRequestBodyProductionEnded();
 }
 
 // premature end of the request or adapted response body production
@@ -307,8 +302,7 @@ ServerStateData::noteBodyProducerAborted(BodyPipe::Pointer bp)
         return;
     }
 #endif
-    if (requestBodySource == bp)
-        handleRequestBodyProducerAborted();
+    handleRequestBodyProducerAborted();
 }
 
 
@@ -654,17 +648,8 @@ ServerStateData::noteAdaptationAnswer(HttpMsg *msg)
 {
     clearAdaptation(adaptedHeadSource); // we do not expect more messages
 
-    if (abortOnBadEntry("entry went bad while waiting for adapted headers")) {
-        // If the adapted response has a body, the ICAP side needs to know
-        // that nobody will consume that body. We will be destroyed upon
-        // return. Tell the ICAP side that it is on its own.
-        HttpReply *rep = dynamic_cast<HttpReply*>(msg);
-        assert(rep);
-        if (rep->body_pipe != NULL)
-            rep->body_pipe->expectNoConsumption();
-
+    if (abortOnBadEntry("entry went bad while waiting for adapted headers"))
         return;
-    }
 
     HttpReply *rep = dynamic_cast<HttpReply*>(msg);
     assert(rep);
@@ -692,63 +677,22 @@ ServerStateData::noteAdaptationQueryAbort(bool final)
     handleAdaptationAborted(!final);
 }
 
-void
-ServerStateData::resumeBodyStorage()
-{
-    if (abortOnBadEntry("store entry aborted while kick producer callback"))
-        return;
-
-    if (!adaptedBodySource)
-        return;
-
-    handleMoreAdaptedBodyAvailable();
-
-    if (adaptedBodySource != NULL && adaptedBodySource->exhausted())
-        endAdaptedBodyConsumption();
-}
-
 // more adapted response body is available
 void
 ServerStateData::handleMoreAdaptedBodyAvailable()
 {
+    const size_t contentSize = adaptedBodySource->buf().contentSize();
+
+    debugs(11,5, HERE << "consuming " << contentSize << " bytes of adapted " <<
+           "response body at offset " << adaptedBodySource->consumedSize());
+
     if (abortOnBadEntry("entry refuses adapted body"))
         return;
 
     assert(entry);
-
-    size_t contentSize = adaptedBodySource->buf().contentSize();
-
-    if (!contentSize)
-        return; // XXX: bytesWanted asserts on zero-size ranges
-
-    const size_t spaceAvailable = entry->bytesWanted(Range<size_t>(0, contentSize), true);
-
-    if (spaceAvailable < contentSize ) {
-        // No or partial body data consuming
-        typedef NullaryMemFunT<ServerStateData> Dialer;
-        AsyncCall::Pointer call = asyncCall(93, 5, "ServerStateData::resumeBodyStorage",
-                                            Dialer(this, &ServerStateData::resumeBodyStorage));
-        entry->deferProducer(call);
-    }
-
-    if (!spaceAvailable)  {
-        debugs(11, 5, HERE << "NOT storing " << contentSize << " bytes of adapted " <<
-               "response body at offset " << adaptedBodySource->consumedSize());
-        return;
-    }
-
-    if (spaceAvailable < contentSize ) {
-        debugs(11, 5, HERE << "postponing storage of " <<
-               (contentSize - spaceAvailable) << " body bytes");
-        contentSize = spaceAvailable;
-    }
-
-    debugs(11,5, HERE << "storing " << contentSize << " bytes of adapted " <<
-           "response body at offset " << adaptedBodySource->consumedSize());
-
     BodyPipeCheckout bpc(*adaptedBodySource);
-    const StoreIOBuffer ioBuf(&bpc.buf, currentOffset, contentSize);
-    currentOffset += ioBuf.length;
+    const StoreIOBuffer ioBuf(&bpc.buf, currentOffset);
+    currentOffset += bpc.buf.size;
     entry->write(ioBuf);
     bpc.buf.consume(contentSize);
     bpc.checkIn();
@@ -758,19 +702,11 @@ ServerStateData::handleMoreAdaptedBodyAvailable()
 void
 ServerStateData::handleAdaptedBodyProductionEnded()
 {
+    stopConsumingFrom(adaptedBodySource);
+
     if (abortOnBadEntry("entry went bad while waiting for adapted body eof"))
         return;
 
-    // end consumption if we consumed everything
-    if (adaptedBodySource != NULL && adaptedBodySource->exhausted())
-        endAdaptedBodyConsumption();
-    // else resumeBodyStorage() will eventually consume the rest
-}
-
-void
-ServerStateData::endAdaptedBodyConsumption()
-{
-    stopConsumingFrom(adaptedBodySource);
     handleAdaptationCompleted();
 }
 
@@ -879,7 +815,7 @@ ServerStateData::adaptOrFinalizeReply()
     // The callback can be called with a NULL service if adaptation is off.
     adaptationAccessCheckPending = Adaptation::AccessCheck::Start(
                                        Adaptation::methodRespmod, Adaptation::pointPreCache,
-                                       originalRequest(), virginReply(), adaptationAclCheckDoneWrapper, this);
+                                       request, virginReply(), adaptationAclCheckDoneWrapper, this);
     debugs(11,5, HERE << "adaptationAccessCheckPending=" << adaptationAccessCheckPending);
     if (adaptationAccessCheckPending)
         return;
@@ -888,9 +824,27 @@ ServerStateData::adaptOrFinalizeReply()
     setFinalReply(virginReply());
 }
 
+/// initializes bodyBytesRead stats if needed and applies delta
+void
+ServerStateData::adjustBodyBytesRead(const int64_t delta)
+{
+    int64_t &bodyBytesRead = originalRequest()->hier.bodyBytesRead;
+
+    // if we got here, do not log a dash even if we got nothing from the server
+    if (bodyBytesRead < 0)
+        bodyBytesRead = 0;
+
+    bodyBytesRead += delta; // supports negative and zero deltas
+
+    // check for overflows ("infinite" response?) and undeflows (a bug)
+    Must(bodyBytesRead >= 0);
+}
+
 void
 ServerStateData::addVirginReplyBody(const char *data, ssize_t len)
 {
+    adjustBodyBytesRead(len);
+
 #if USE_ADAPTATION
     assert(!adaptationAccessCheckPending); // or would need to buffer while waiting
     if (startedAdaptation) {
