@@ -123,10 +123,6 @@ public:
 
     void setNoDelay (bool const);
     bool modifiedSince(HttpRequest * request) const;
-    /// has ETag matching at least one of the If-Match etags
-    bool hasIfMatchEtag(const HttpRequest &request) const;
-    /// has ETag matching at least one of the If-None-Match etags
-    bool hasIfNoneMatchEtag(const HttpRequest &request) const;
 
     /** What store does this entry belong too ? */
     virtual RefCount<Store> store() const;
@@ -139,15 +135,15 @@ public:
     time_t expires;
     time_t lastmod;
     uint64_t swap_file_sz;
-    uint16_t refcount;
-    uint16_t flags;
+    u_short refcount;
+    u_short flags;
     /* END OF ON-DISK STORE_META_STD */
 
     sfileno swap_filen:25;
 
     sdirno swap_dirn:7;
 
-    unsigned short lock_count;		/* Assume < 65536! */
+    u_short lock_count;		/* Assume < 65536! */
 
     mem_status_t mem_status:3;
 
@@ -189,23 +185,10 @@ public:
     virtual void lock();
     virtual void release();
 
-#if USE_ADAPTATION
-    /// call back producer when more buffer space is available
-    void deferProducer(AsyncCall::Pointer &producer);
-    /// calls back producer registered with deferProducer
-    void kickProducer();
-#endif
-
 private:
     static MemAllocator *pool;
 
-#if USE_ADAPTATION
-    /// producer callback registered with deferProducer
-    AsyncCall::Pointer deferredProducer;
-#endif
-
     bool validLength() const;
-    bool hasOneOfEtags(const String &reqETags, const bool allowWeakMatch) const;
 };
 
 /// \ingroup StoreAPI
@@ -290,10 +273,10 @@ public:
      * The maximum size the store will support in normal use. Inaccuracy is permitted,
      * but may throw estimates for memory etc out of whack.
      */
-    virtual uint64_t maxSize() const = 0;
+    virtual size_t maxSize() const = 0;
 
     /** The minimum size the store will shrink to via normal housekeeping */
-    virtual uint64_t minSize() const = 0;
+    virtual size_t minSize() const = 0;
 
     /**
      * Output stats to the provided store entry.
@@ -394,7 +377,10 @@ extern FREE destroyStoreEntry;
  */
 SQUIDCEXTERN void packerToStoreInit(Packer * p, StoreEntry * e);
 
-#ifdef _USE_INLINE_
+/// \ingroup StoreAPI
+SQUIDCEXTERN void storeGetMemSpace(int size);
+
+#if _USE_INLINE_
 #include "Store.cci"
 #endif
 
