@@ -34,8 +34,8 @@
  */
 
 #include "squid.h"
+#include "base/TextException.h"
 #include "Store.h"
-#include "TextException.h"
 
 int
 String::psize() const
@@ -150,6 +150,7 @@ String::clean()
     PROF_start(StringClean);
     assert(this);
 
+    /* TODO if mempools has already closed this will FAIL!! */
     if (defined())
         memFreeString(size_, buf_);
 
@@ -491,6 +492,6 @@ String::rfind(char const ch) const
 
 
 
-#ifndef _USE_INLINE_
+#if !_USE_INLINE_
 #include "String.cci"
 #endif
