@@ -5,13 +5,13 @@
 #include "HttpRequest.h"
 #include "HttpReply.h"
 #include "BodyPipe.h"
-#include "TextException.h"
 #include <libecap/common/names.h>
 #include <libecap/common/area.h>
 #include <libecap/common/version.h>
 #include "adaptation/ecap/MessageRep.h"
 #include "adaptation/ecap/XactionRep.h"
 #include "adaptation/ecap/Host.h" /* for protocol constants */
+#include "base/TextException.h"
 
 /* HeaderRep */
 
@@ -48,9 +48,6 @@ Adaptation::Ecap::HeaderRep::add(const Name &name, const Value &value)
     HttpHeaderEntry *e = new HttpHeaderEntry(squidId, name.image().c_str(),
             value.toString().c_str());
     theHeader.addEntry(e);
-
-    if (squidId == HDR_CONTENT_LENGTH)
-        theMessage.content_length = theHeader.getInt64(HDR_CONTENT_LENGTH);
 }
 
 void
@@ -61,9 +58,6 @@ Adaptation::Ecap::HeaderRep::removeAny(const Name &name)
         theHeader.delByName(name.image().c_str());
     else
         theHeader.delById(squidId);
-
-    if (squidId == HDR_CONTENT_LENGTH)
-        theMessage.content_length = theHeader.getInt64(HDR_CONTENT_LENGTH);
 }
 
 libecap::Area
@@ -150,8 +144,6 @@ Adaptation::Ecap::FirstLineRep::protocol() const
         return protocolInternal;
     case PROTO_ICY:
         return Name();
-    case PROTO_COAP:
-    case PROTO_COAPS: // use 'unknown'/none until libecap supports coap:// and coaps://
     case PROTO_NONE:
         return Name();
 
@@ -201,10 +193,8 @@ Adaptation::Ecap::RequestLineRep::uri(const Area &aUri)
 Adaptation::Ecap::RequestLineRep::Area
 Adaptation::Ecap::RequestLineRep::uri() const
 {
-    const char *fullUrl = urlCanonical(&theMessage);
-    Must(fullUrl);
-    // optimize: avoid copying by having an Area::Detail that locks theMessage
-    return Area::FromTempBuffer(fullUrl, strlen(fullUrl));
+    return Area::FromTempBuffer(theMessage.urlpath.rawBuf(),
+                                theMessage.urlpath.size());
 }
 
 void
