@@ -1,4 +1,3 @@
-
 /*
  * $Id$
  *
@@ -39,7 +38,21 @@
 #include "Store.h"
 #include "fde.h"
 
-#ifdef USE_POLL
+#if USE_POLL
+
+#if HAVE_POLL_H
+#include <poll.h>
+#endif
+
+/* Needed for poll() on Linux at least */
+#if USE_POLL
+#ifndef POLLRDNORM
+#define POLLRDNORM POLLIN
+#endif
+#ifndef POLLWRNORM
+#define POLLWRNORM POLLOUT
+#endif
+#endif
 
 static int MAX_POLL_TIME = 1000;	/* see also comm_quick_poll_required() */
 
@@ -402,7 +415,7 @@ comm_select(int msec)
          * Note that this will only ever trigger when there are no log files
          * and stdout/err/in are all closed too.
          */
-        if (nfds == 0 && npending == 0) {
+        if (nfds == 0 && !npending) {
             if (shutting_down)
                 return COMM_SHUTDOWN;
             else
@@ -416,7 +429,7 @@ comm_select(int msec)
             ++statCounter.select_loops;
             PROF_stop(comm_poll_normal);
 
-            if (num >= 0 || npending > 0)
+            if (num >= 0 || npending >= 0)
                 break;
 
             if (ignoreErrno(errno))
