@@ -36,7 +36,7 @@
 #include "CacheManager.h"
 #include "SquidTime.h"
 
-#ifdef USE_SELECT_WIN32
+#if USE_SELECT_WIN32
 #include "Store.h"
 #include "fde.h"
 
@@ -175,7 +175,10 @@ fdIsIcp(int fd)
 static int
 fdIsDns(int fd)
 {
-    if (fd == DnsSocket)
+    if (fd == DnsSocketA)
+        return 1;
+
+    if (fd == DnsSocketB)
         return 1;
 
     return 0;
@@ -647,14 +650,18 @@ static void
 comm_select_dns_incoming(void)
 {
     int nfds = 0;
-    int fds[2];
+    int fds[3];
     int nevents;
     dns_io_events = 0;
 
-    if (DnsSocket < 0)
+    if (DnsSocketA < 0 && DnsSocketB < 0)
         return;
 
-    fds[nfds++] = DnsSocket;
+    if (DnsSocketA >= 0)
+        fds[nfds++] = DnsSocketA;
+
+    if (DnsSocketB >= 0)
+        fds[nfds++] = DnsSocketB;
 
     nevents = comm_check_incoming_select_handlers(nfds, fds);
 
