@@ -31,19 +31,16 @@
  */
 
 #include "config.h"
+#include "compat/inet_ntop.h"
+#include "compat/getaddrinfo.h"
+#include "compat/getnameinfo.h"
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
 #if HAVE_STDIO_H
 #include <stdio.h>
 #endif
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
 #if HAVE_CTYPE_H
 #include <ctype.h>
 #endif
@@ -64,8 +61,7 @@
 #if HAVE_MEMORY_H
 #include <memory.h>
 #endif
-#if HAVE_NETDB_H && !defined(_SQUID_NETDB_H_)	/* protect NEXTSTEP */
-#define _SQUID_NETDB_H_
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_PWD_H
@@ -80,12 +76,6 @@
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
-#if HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#if HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>	/* needs sys/time.h above it */
-#endif
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -107,24 +97,21 @@
 #if HAVE_LIBC_H
 #include <libc.h>
 #endif
-#ifdef HAVE_SYS_SYSCALL_H
+#if HAVE_SYS_SYSCALL_H
 #include <sys/syscall.h>
 #endif
-#ifdef HAVE_STRING_H
+#if HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_STRINGS_H
+#if HAVE_STRINGS_H
 #include <strings.h>
 #endif
 #if HAVE_BSTRING_H
 #include <bstring.h>
 #endif
-#ifdef HAVE_CRYPT_H
+#if HAVE_CRYPT_H
 #include <crypt.h>
 #endif
-#if HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
 #if HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -163,7 +150,7 @@ usage: dnsserver -Dhv -s nameserver
  \endverbatim
  */
 
-#include "ip/IpAddress.h"
+#include "ip/Address.h"
 
 #if LIBRESOLV_DNS_TTL_HACK
 /// \ingroup dnsserver
@@ -192,7 +179,8 @@ lookup(const char *buf)
     int ttl = 0;
     int retry = 0;
     unsigned int i = 0;
-    char ntoabuf[256];
+    Ip::Address ipa;
+    char ntoabuf[MAX_IPSTRLEN];
     struct addrinfo hints;
     struct addrinfo *AI = NULL;
     struct addrinfo *aiptr = NULL;
@@ -207,29 +195,16 @@ lookup(const char *buf)
         return;
     }
 
-    /* check if it's already an IP address in text form. */
-    memset(&hints, '\0', sizeof(struct addrinfo));
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_flags = AI_NUMERICHOST; // only succeed if its numeric.
-    const bool isDomain = (getaddrinfo(buf,NULL,&hints,&AI) != 0);
-
-    // reset for real lookup
-    if (AI != NULL) {
-        freeaddrinfo(AI);
-        AI = NULL;
-    }
-
-    // resolve the address/name
+    /* setup 'hints' for the system lookup */
     memset(&hints, '\0', sizeof(struct addrinfo));
     hints.ai_family = AF_UNSPEC;
     hints.ai_flags = AI_CANONNAME;
-    for (;;) {
-        if (AI != NULL) {
-            xfreeaddrinfo(AI);
-            AI = NULL;
-        }
 
-        if ( 0 == (res = xgetaddrinfo(buf,NULL,&hints,&AI)) )
+    for (;;) {
+        freeaddrinfo(AI);
+        AI = NULL;
+
+        if ( 0 == (res = getaddrinfo(buf,NULL,&hints,&AI)) )
             break;
 
         if (res != EAI_AGAIN)
@@ -241,7 +216,9 @@ lookup(const char *buf)
         sleep(1);
     }
 
-    if (isDomain) {
+    /* check if it's already an IP address in text form. */
+    ipa = buf;
+    if ( ipa.IsAnyAddr() ) {
         /* its a domain name. Use the forward-DNS lookup already done */
 
         if (res == 0) {
@@ -257,7 +234,7 @@ lookup(const char *buf)
             i = 0;
             aiptr = AI;
             while (NULL != aiptr && 32 >= i) {
-                memset(ntoabuf, 0, sizeof(ntoabuf));
+                memset(ntoabuf, 0, MAX_IPSTRLEN);
 
                 /* getaddrinfo given a host has a nasty tendency to return duplicate addr's */
                 /* BUT sorted fortunately, so we can drop most of them easily */
@@ -275,10 +252,10 @@ lookup(const char *buf)
                 /* annoying inet_ntop breaks the nice code by requiring the in*_addr */
                 switch (aiptr->ai_family) {
                 case AF_INET:
-                    xinet_ntop(aiptr->ai_family, &((struct sockaddr_in*)aiptr->ai_addr)->sin_addr, ntoabuf, sizeof(ntoabuf));
+                    inet_ntop(aiptr->ai_family, &((struct sockaddr_in*)aiptr->ai_addr)->sin_addr, ntoabuf, MAX_IPSTRLEN);
                     break;
                 case AF_INET6:
-                    xinet_ntop(aiptr->ai_family, &((struct sockaddr_in6*)aiptr->ai_addr)->sin6_addr, ntoabuf, sizeof(ntoabuf));
+                    inet_ntop(aiptr->ai_family, &((struct sockaddr_in6*)aiptr->ai_addr)->sin6_addr, ntoabuf, MAX_IPSTRLEN);
                     break;
                 default:
                     aiptr = aiptr->ai_next;
@@ -299,7 +276,7 @@ lookup(const char *buf)
          */
         if (NULL != AI && NULL != AI->ai_addr) {
             for (;;) {
-                if ( 0 == (res = xgetnameinfo(AI->ai_addr, AI->ai_addrlen, ntoabuf, sizeof(ntoabuf), NULL,0,0)) )
+                if ( 0 == (res = getnameinfo(AI->ai_addr, AI->ai_addrlen, ntoabuf, MAX_IPSTRLEN, NULL,0,0)) )
                     break;
 
                 if (res != EAI_AGAIN)
@@ -335,26 +312,25 @@ lookup(const char *buf)
         break;
 
     case EAI_FAIL:
-        printf("$fail DNS Domain/IP '%s' does not exist: %s.\n", buf, xgai_strerror(res));
+        printf("$fail DNS Domain/IP '%s' does not exist: %s.\n", buf, gai_strerror(res));
         break;
 
 #if defined(EAI_NODATA) || defined(EAI_NONAME)
-#ifdef EAI_NODATA
+#if EAI_NODATA
         /* deprecated. obsolete on some OS */
     case EAI_NODATA:
 #endif
-#ifdef EAI_NONAME
+#if EAI_NONAME
     case EAI_NONAME:
 #endif
-        printf("$fail DNS Domain/IP '%s' exists without any FQDN/IPs: %s.\n", buf, xgai_strerror(res));
+        printf("$fail DNS Domain/IP '%s' exists without any FQDN/IPs: %s.\n", buf, gai_strerror(res));
         break;
 #endif
     default:
-        printf("$fail A system error occured looking up Domain/IP '%s': %s.\n", buf, xgai_strerror(res));
+        printf("$fail A system error occured looking up Domain/IP '%s': %s.\n", buf, gai_strerror(res));
     }
 
-    if (AI != NULL)
-        xfreeaddrinfo(AI);
+    freeaddrinfo(AI);
 }
 
 /**
@@ -395,9 +371,12 @@ squid_res_setservers(int reset)
     /* Gone again on FreeBSD 6.2 along with _res_ext itself in any form. */
     int ns6count = 0;
 #endif
-#if HAVE_RES_INIT && defined(_SQUID_RES_NSADDR_LIST)
+#if HAVE_RES_INIT
+    Ip::Address ipa;
+#ifdef _SQUID_RES_NSADDR_LIST
     extern char *optarg;
 #endif
+#endif
 
 #if HAVE_RES_INIT && (defined(_SQUID_RES_NSADDR_LIST) || defined(_SQUID_RES_NSADDR6_LIST))
 
@@ -422,41 +401,35 @@ squid_res_setservers(int reset)
      *
      *  BUT, even if _res.nsaddrs is memset to NULL, it resolves IFF IPv6 set in _ext.
      *
-     *  SO, am splitting the IPv4/v6 into the seperate _res fields
+     *  SO, am splittig the IPv4/v6 into the seperate _res fields
      *      and making nscount a total of IPv4+IPv6 /w nscount6 the IPv6 sub-counter
      *	ie. nscount = count(NSv4)+count(NSv6) & nscount6 = count(NSv6)
      *
      * If ANYONE knows better please let us know.
      */
-    struct addrinfo hints;
-    memset(&hints, '\0', sizeof(struct addrinfo));
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_flags = AI_NUMERICHOST; // prevent repeated DNS lookups!
-    struct addrinfo *AI = NULL;
-    if ( getaddrinfo(optarg, NULL, &hints, &AI) != 0) {
+    if ( !(ipa = optarg) ) {
         fprintf(stderr, "%s appears to be a bad nameserver FQDN/IP.\n",optarg);
-    } else if ( AI->ai_family == AF_INET ) {
+    } else if ( ipa.IsIPv4() ) {
         if (_SQUID_RES_NSADDR_COUNT == MAXNS) {
             fprintf(stderr, "Too many -s options, only %d are allowed\n", MAXNS);
-        } else {
-            _SQUID_RES_NSADDR_LIST[_SQUID_RES_NSADDR_COUNT] = _SQUID_RES_NSADDR_LIST[0];
-            memcpy(&_SQUID_RES_NSADDR_LIST[_SQUID_RES_NSADDR_COUNT++].sin_addr, &((struct sockaddr_in*)AI->ai_addr)->sin_addr, sizeof(struct in_addr));
+            return;
         }
-    } else if ( AI->ai_family == AF_INET6 ) {
+        _SQUID_RES_NSADDR_LIST[_SQUID_RES_NSADDR_COUNT] = _SQUID_RES_NSADDR_LIST[0];
+        ipa.GetInAddr(_SQUID_RES_NSADDR_LIST[_SQUID_RES_NSADDR_COUNT++].sin_addr);
+    } else if ( ipa.IsIPv6() ) {
 #if USE_IPV6 && defined(_SQUID_RES_NSADDR6_LIST)
         /* because things NEVER seem to resolve in tests without _res.nscount being a total. */
         if (_SQUID_RES_NSADDR_COUNT == MAXNS) {
             fprintf(stderr, "Too many -s options, only %d are allowed\n", MAXNS);
-        } else {
-            _SQUID_RES_NSADDR_COUNT++;
-            memcpy(&_SQUID_RES_NSADDR6_LIST(_SQUID_RES_NSADDR6_COUNT++), &((struct sockaddr_in6*)AI->ai_addr)->sin6_addr, sizeof(struct in6_addr));
+            return;
         }
+        _SQUID_RES_NSADDR_COUNT++;
+
+        ipa.GetInAddr(_SQUID_RES_NSADDR6_LIST(_SQUID_RES_NSADDR6_COUNT++));
 #else
         fprintf(stderr, "IPv6 nameservers not supported on this resolver\n");
 #endif
     }
-    if (AI != NULL)
-        freeaddrinfo(AI);
 
 #else /* !HAVE_RES_INIT || !defined(_SQUID_RES_NSADDR_LIST) */
 
