@@ -45,6 +45,7 @@
 #include "SquidMath.h"
 #include "SquidTime.h"
 #include "SwapDir.h"
+#include "swap_log_op.h"
 
 int UFSSwapDir::NumberOfUFSDirs = 0;
 int *UFSSwapDir::UFSDirToGlobalDirMapping = NULL;
@@ -71,11 +72,14 @@ UFSSwapDir::canStore(StoreEntry const &e)const
 void
 UFSSwapDir::parseSizeL1L2()
 {
-    int i = GetInteger();
-    if (i <= 0)
-        fatal("UFSSwapDir::parseSizeL1L2: invalid size value");
+    int i;
+    int size;
 
-    size_t size = i << 10;		/* Mbytes to kbytes */
+    i = GetInteger();
+    size = i << 10;		/* Mbytes to kbytes */
+
+    if (size <= 0)
+        fatal("UFSSwapDir::parseSizeL1L2: invalid size value");
 
     /* just reconfigure it */
     if (reconfiguring) {
@@ -311,10 +315,10 @@ UFSSwapDir::statfs(StoreEntry & sentry) const
     int x;
     storeAppendPrintf(&sentry, "First level subdirectories: %d\n", l1);
     storeAppendPrintf(&sentry, "Second level subdirectories: %d\n", l2);
-    storeAppendPrintf(&sentry, "Maximum Size: %" PRIu64 " KB\n", max_size);
-    storeAppendPrintf(&sentry, "Current Size: %" PRIu64 " KB\n", cur_size);
+    storeAppendPrintf(&sentry, "Maximum Size: %d KB\n", max_size);
+    storeAppendPrintf(&sentry, "Current Size: %d KB\n", cur_size);
     storeAppendPrintf(&sentry, "Percent Used: %0.2f%%\n",
-                      (double)(100.0 * cur_size) / (double)max_size);
+                      100.0 * cur_size / max_size);
     storeAppendPrintf(&sentry, "Filemap bits in use: %d of %d (%d%%)\n",
                       map->n_files_in_map, map->max_n_files,
                       Math::intPercent(map->n_files_in_map, map->max_n_files));
@@ -377,7 +381,7 @@ UFSSwapDir::maintain()
     walker = repl->PurgeInit(repl, max_scan);
 
     while (1) {
-        if (cur_size < minSize())
+        if (cur_size < (int) minSize()) /* cur_size should be unsigned */
             break;
 
         if (removed >= max_remove)
@@ -480,9 +484,9 @@ UFSSwapDir::mapBitAllocate()
 char *
 UFSSwapDir::swapSubDir(int subdirn)const
 {
-    LOCAL_ARRAY(char, fullfilename, SQUID_MAXPATHLEN);
+    LOCAL_ARRAY(char, fullfilename, MAXPATHLEN);
     assert(0 <= subdirn && subdirn < l1);
-    snprintf(fullfilename, SQUID_MAXPATHLEN, "%s/%02X", path, subdirn);
+    snprintf(fullfilename, MAXPATHLEN, "%s/%02X", path, subdirn);
     return fullfilename;
 }
 
@@ -585,13 +589,13 @@ UFSSwapDir::createSwapSubDirs()
 char *
 UFSSwapDir::logFile(char const *ext) const
 {
-    LOCAL_ARRAY(char, lpath, SQUID_MAXPATHLEN);
-    LOCAL_ARRAY(char, pathtmp, SQUID_MAXPATHLEN);
+    LOCAL_ARRAY(char, lpath, MAXPATHLEN);
+    LOCAL_ARRAY(char, pathtmp, MAXPATHLEN);
     LOCAL_ARRAY(char, digit, 32);
     char *pathtmp2;
 
     if (Config.Log.swap) {
-        xstrncpy(pathtmp, path, SQUID_MAXPATHLEN - 64);
+        xstrncpy(pathtmp, path, MAXPATHLEN - 64);
         pathtmp2 = pathtmp;
 
         while ((pathtmp2 = strchr(pathtmp2, '/')) != NULL)
@@ -601,15 +605,15 @@ UFSSwapDir::logFile(char const *ext) const
             pathtmp[strlen(pathtmp) - 1] = '\0';
 
         for (pathtmp2 = pathtmp; *pathtmp2 == '.'; pathtmp2++);
-        snprintf(lpath, SQUID_MAXPATHLEN - 64, Config.Log.swap, pathtmp2);
+        snprintf(lpath, MAXPATHLEN - 64, Config.Log.swap, pathtmp2);
 
-        if (strncmp(lpath, Config.Log.swap, SQUID_MAXPATHLEN - 64) == 0) {
+        if (strncmp(lpath, Config.Log.swap, MAXPATHLEN - 64) == 0) {
             strcat(lpath, ".");
             snprintf(digit, 32, "%02d", index);
             strncat(lpath, digit, 3);
         }
     } else {
-        xstrncpy(lpath, path, SQUID_MAXPATHLEN - 64);
+        xstrncpy(lpath, path, MAXPATHLEN - 64);
         strcat(lpath, "/swap.state");
     }
 
@@ -683,8 +687,8 @@ UFSSwapDir::addDiskRestore(const cache_key * key,
                            time_t timestamp,
                            time_t lastref,
                            time_t lastmod,
-                           u_int32_t refcount,
-                           u_int16_t newFlags,
+                           uint32_t refcount,
+                           uint16_t newFlags,
                            int clean)
 {
     StoreEntry *e = NULL;
@@ -733,15 +737,14 @@ UFSSwapDir::closeTmpSwapLog()
     file_close(swaplog_fd);
 
     if (xrename(new_path, swaplog_path) < 0) {
-        debugs(50, DBG_IMPORTANT, "ERROR: " << swaplog_path << ": " << xstrerror());
-        fatalf("Failed to rename log file %s to %s.new", swaplog_path, swaplog_path);
+        fatal("commonUfsDirCloseTmpSwapLog: rename failed");
     }
 
     fd = file_open(swaplog_path, O_WRONLY | O_CREAT | O_BINARY);
 
     if (fd < 0) {
-        debugs(50, DBG_IMPORTANT, "ERROR: " << swaplog_path << ": " << xstrerror());
-        fatalf("Failed to open swap log %s", swaplog_path);
+        debugs(50, 1, "" << swaplog_path << ": " << xstrerror());
+        fatal("commonUfsDirCloseTmpSwapLog: Failed to open swap log.");
     }
 
     safe_free(swaplog_path);
@@ -750,6 +753,13 @@ UFSSwapDir::closeTmpSwapLog()
     debugs(47, 3, "Cache Dir #" << index << " log opened on FD " << fd);
 }
 
+static void
+FreeHeader(void *address)
+{
+    StoreSwapLogHeader *anObject = static_cast <StoreSwapLogHeader *>(address);
+    delete anObject;
+}
+
 FILE *
 UFSSwapDir::openTmpSwapLog(int *clean_flag, int *zero_flag)
 {
@@ -762,6 +772,7 @@ UFSSwapDir::openTmpSwapLog(int *clean_flag, int *zero_flag)
     struct stat clean_sb;
     FILE *fp;
     int fd;
+    StoreSwapLogHeader *head;
 
     if (::stat(swaplog_path, &log_sb) < 0) {
         debugs(47, 1, "Cache Dir #" << index << ": No log file");
@@ -787,18 +798,10 @@ UFSSwapDir::openTmpSwapLog(int *clean_flag, int *zero_flag)
 
     swaplog_fd = fd;
 
-    {
-        const StoreSwapLogHeader header;
-        MemBuf buf;
-        buf.init(header.record_size, header.record_size);
-        buf.append(reinterpret_cast<const char*>(&header), sizeof(header));
-        // Pad to keep in sync with UFSSwapDir::writeCleanStart().
-        // TODO: When MemBuf::spaceSize() is fixed not to subtract one,
-        // memset() space() with zeroes and use spaceSize() below.
-        buf.appended(static_cast<size_t>(header.record_size) - sizeof(header));
-        file_write(swaplog_fd, -1, buf.content(), buf.contentSize(),
-                   NULL, NULL, buf.freeFunc());
-    }
+    head = new StoreSwapLogHeader;
+
+    file_write(swaplog_fd, -1, head, head->record_size,
+               NULL, NULL, FreeHeader);
 
     /* open a read-only stream of the old log */
     fp = fopen(swaplog_path, "rb");
@@ -883,7 +886,6 @@ UFSSwapDir::writeCleanStart()
     state->outbuf_offset = 0;
     /*copy the header */
     xmemcpy(state->outbuf, &header, sizeof(StoreSwapLogHeader));
-    // Leave a gap to keep in sync with UFSSwapDir::openTmpSwapLog().
     state->outbuf_offset += header.record_size;
 
     state->walker = repl->WalkInit(repl);
@@ -1065,6 +1067,7 @@ UFSSwapDir::DirClean(int swap_index)
 {
     DIR *dir_pointer = NULL;
 
+    struct dirent *de = NULL;
     LOCAL_ARRAY(char, p1, MAXPATHLEN + 1);
     LOCAL_ARRAY(char, p2, MAXPATHLEN + 1);
 
@@ -1084,7 +1087,7 @@ UFSSwapDir::DirClean(int swap_index)
     D1 = (swap_index / N0) % N1;
     N2 = SD->l2;
     D2 = ((swap_index / N0) / N1) % N2;
-    snprintf(p1, SQUID_MAXPATHLEN, "%s/%02X/%02X",
+    snprintf(p1, MAXPATHLEN, "%s/%02X/%02X",
              SD->path, D1, D2);
     debugs(36, 3, "storeDirClean: Cleaning directory " << p1);
     dir_pointer = opendir(p1);
@@ -1108,7 +1111,6 @@ UFSSwapDir::DirClean(int swap_index)
         return 0;
     }
 
-    dirent_t *de;
     while ((de = readdir(dir_pointer)) != NULL && k < 20) {
         if (sscanf(de->d_name, "%X", &swapfileno) != 1)
             continue;
@@ -1324,8 +1326,8 @@ UFSSwapDir::replacementRemove(StoreEntry * e)
 void
 UFSSwapDir::dump(StoreEntry & entry) const
 {
-    storeAppendPrintf(&entry, " %" PRIu64 " %d %d",
-                      (max_size >> 10),
+    storeAppendPrintf(&entry, " %d %d %d",
+                      max_size >> 10,
                       l1,
                       l2);
     dumpOptions(&entry);
@@ -1334,7 +1336,7 @@ UFSSwapDir::dump(StoreEntry & entry) const
 char *
 UFSSwapDir::fullPath(sfileno filn, char *fullpath) const
 {
-    LOCAL_ARRAY(char, fullfilename, SQUID_MAXPATHLEN);
+    LOCAL_ARRAY(char, fullfilename, MAXPATHLEN);
     int L1 = l1;
     int L2 = l2;
 
@@ -1343,7 +1345,7 @@ UFSSwapDir::fullPath(sfileno filn, char *fullpath) const
 
     fullpath[0] = '\0';
 
-    snprintf(fullpath, SQUID_MAXPATHLEN, "%s/%02X/%02X/%08X",
+    snprintf(fullpath, MAXPATHLEN, "%s/%02X/%02X/%08X",
              path,
              ((filn / L2) / L2) % L1,
              (filn / L2) % L2,
