@@ -102,7 +102,7 @@ extern "C" {
     extern int theOutIcpConnection;	/* -1 */
     extern int DnsSocketA;		/* -1 */
     extern int DnsSocketB;		/* -1 */
-#ifdef SQUID_SNMP
+#if SQUID_SNMP
 
     extern int theInSnmpConnection;	/* -1 */
     extern int theOutSnmpConnection;	/* -1 */
@@ -114,19 +114,13 @@ extern "C" {
 
     extern struct acl_deny_info_list *DenyInfoList;	/* NULL */
 
-//MOVED:icp_v2.cc    extern IpAddress theOutICPAddr;
-//MOVED:snmp_core.cc    extern IpAddress theOutSNMPAddr;
-
     extern struct timeval squid_start;
+    extern int starting_up;	/* 1 */
     extern int shutting_down;	/* 0 */
     extern int reconfiguring;	/* 0 */
     extern unsigned long store_swap_size;	/* 0 */
     extern time_t hit_only_mode_until;	/* 0 */
     extern StatCounters statCounter;
-    extern char *err_type_str[];
-    extern char *icp_opcode_str[];
-    extern char *swap_log_op_str[];
-    extern char *lookup_t_str[];
     extern double request_failure_ratio;	/* 0.0 */
     extern int store_hash_buckets;	/* 0 */
     extern hash_table *store_table;	/* NULL */
@@ -138,7 +132,7 @@ extern "C" {
     extern const char *StoreDigestMimeStr;	/* "application/cache-digest" */
 
     extern const char *MultipartMsgBoundaryStr;	/* "Unique-Squid-Separator" */
-#if HTTP_VIOLATIONS
+#if USE_HTTP_VIOLATIONS
 
     extern int refresh_nocache_hack;	/* 0 */
 #endif
@@ -172,13 +166,14 @@ extern "C" {
     extern int ssl_ex_index_server;	/* -1 */
     extern int ssl_ctx_ex_index_dont_verify_domain; /* -1 */
     extern int ssl_ex_index_cert_error_check;	/* -1 */
-    extern int ssl_ex_index_ssl_error_detail;      /* -1 */
 
     extern const char *external_acl_message;      /* NULL */
     extern int opt_send_signal;	/* -1 */
     extern int opt_no_daemon; /* 0 */
-    extern int opt_parse_cfg_only; /* 0 */
 
+/// current Squid process number (e.g., 4).
+/// Zero for SMP-unaware code and in no-SMP mode.
+    extern int KidIdentifier; /* 0 */
 
 #ifdef __cplusplus
 }
