@@ -45,7 +45,7 @@
 #include "SquidTime.h"
 #include "Store.h"
 #include "icmp/net_db.h"
-#include "ip/IpAddress.h"
+#include "ip/Address.h"
 #include "ip/tools.h"
 
 /* count mcast group peers every 15 minutes */
@@ -68,13 +68,13 @@ static void peerCountMcastPeersStart(void *data);
 static void peerCountMcastPeersSchedule(peer * p, time_t when);
 static IRCB peerCountHandleIcpReply;
 
-static void neighborIgnoreNonPeer(const IpAddress &, icp_opcode);
+static void neighborIgnoreNonPeer(const Ip::Address &, icp_opcode);
 static OBJH neighborDumpPeers;
 static OBJH neighborDumpNonPeers;
 static void dump_peers(StoreEntry * sentry, peer * peers);
 
 static icp_common_t echo_hdr;
-static unsigned short echo_port;
+static u_short echo_port;
 
 static int NLateReplies = 0;
 static peer *first_ping = NULL;
@@ -96,7 +96,7 @@ neighborTypeStr(const peer * p)
 
 
 peer *
-whichPeer(const IpAddress &from)
+whichPeer(const Ip::Address &from)
 {
     int j;
 
@@ -167,11 +167,6 @@ peerAllowedToUse(const peer * p, HttpRequest * request)
             return 0;
     }
 
-    // CONNECT requests are proxy requests. Not to be forwarded to origin servers.
-    // Unless the destination port matches, in which case we MAY perform a 'DIRECT' to this peer.
-    if (p->options.originserver && request->method == METHOD_CONNECT && request->port != p->in_addr.GetPort())
-        return 0;
-
     if (p->peer_domain == NULL && p->access == NULL)
         return do_ping;
 
@@ -214,19 +209,22 @@ peerAllowedToUse(const peer * p, HttpRequest * request)
 static int
 peerWouldBePinged(const peer * p, HttpRequest * request)
 {
-    if (p->icp.port == 0)
+    if (!peerAllowedToUse(p, request))
         return 0;
 
     if (p->options.no_query)
         return 0;
 
+    if (p->options.background_ping && (squid_curtime - p->stats.last_query < Config.backgroundPingRate))
+        return 0;
+
     if (p->options.mcast_responder)
         return 0;
 
     if (p->n_addresses == 0)
         return 0;
 
-    if (p->options.background_ping && (squid_curtime - p->stats.last_query < Config.backgroundPingRate))
+    if (p->icp.port == 0)
         return 0;
 
     /* the case below seems strange, but can happen if the
@@ -235,9 +233,6 @@ peerWouldBePinged(const peer * p, HttpRequest * request)
         if (!request->flags.hierarchical)
             return 0;
 
-    if (!peerAllowedToUse(p, request))
-        return 0;
-
     /* Ping dead peers every timeout interval */
     if (squid_curtime - p->stats.last_query > Config.Timeout.deadPeer)
         return 1;
@@ -252,16 +247,16 @@ peerWouldBePinged(const peer * p, HttpRequest * request)
 int
 peerHTTPOkay(const peer * p, HttpRequest * request)
 {
-    if (p->max_conn)
-        if (p->stats.conn_open >= p->max_conn)
-            return 0;
-
     if (!peerAllowedToUse(p, request))
         return 0;
 
     if (!neighborUp(p))
         return 0;
 
+    if (p->max_conn)
+        if (p->stats.conn_open >= p->max_conn)
+            return 0;
+
     return 1;
 }
 
@@ -325,7 +320,7 @@ getRoundRobinParent(HttpRequest * request)
             if (p->weight == q->weight) {
                 if (q->rr_count < p->rr_count)
                     continue;
-            } else if ( ((double) q->rr_count / q->weight) < ((double) p->rr_count / p->weight)) {
+            } else if ( (double) q->rr_count / q->weight < (double) p->rr_count / p->weight) {
                 continue;
             }
         }
@@ -477,6 +472,33 @@ getDefaultParent(HttpRequest * request)
     return NULL;
 }
 
+/*
+ * XXX DW thinks this function is equivalent to/redundant with
+ * getFirstUpParent().  peerHTTPOkay() only returns true if the
+ * peer is UP anyway, so this function would not return a
+ * DOWN parent.
+ */
+peer *
+getAnyParent(HttpRequest * request)
+{
+    peer *p = NULL;
+
+    for (p = Config.peers; p; p = p->next) {
+        if (neighborType(p, request) != PEER_PARENT)
+            continue;
+
+        if (!peerHTTPOkay(p, request))
+            continue;
+
+        debugs(15, 3, "getAnyParent: returning " << p->host);
+
+        return p;
+    }
+
+    debugs(15, 3, "getAnyParent: returning NULL");
+    return NULL;
+}
+
 peer *
 getNextPeer(peer * p)
 {
@@ -533,7 +555,7 @@ neighborsRegisterWithCacheManager()
 void
 neighbors_init(void)
 {
-    IpAddress nul;
+    Ip::Address nul;
     struct addrinfo *AI = NULL;
     struct servent *sep = NULL;
     const char *me = getMyHostname();
@@ -586,7 +608,7 @@ neighbors_init(void)
         nul = *AI;
         nul.GetInAddr( *((struct in_addr*)&echo_hdr.shostid) );
         sep = getservbyname("echo", "udp");
-        echo_port = sep ? ntohs((unsigned short) sep->s_port) : 7;
+        echo_port = sep ? ntohs((u_short) sep->s_port) : 7;
     }
 
     first_ping = Config.peers;
@@ -805,6 +827,7 @@ neighborsDigestSelect(HttpRequest * request)
     peer *best_p = NULL;
 #if USE_CACHE_DIGESTS
 
+    const cache_key *key;
     int best_rtt = 0;
     int choice_count = 0;
     int ichoice_count = 0;
@@ -815,7 +838,7 @@ neighborsDigestSelect(HttpRequest * request)
     if (!request->flags.hierarchical)
         return NULL;
 
-    storeKeyPublicByRequest(request);
+    key = storeKeyPublicByRequest(request);
 
     for (i = 0, p = first_ping; i++ < Config.npeers; p = p->next) {
         lookup_t lookup;
@@ -866,18 +889,13 @@ void
 peerNoteDigestLookup(HttpRequest * request, peer * p, lookup_t lookup)
 {
 #if USE_CACHE_DIGESTS
-
     if (p)
         strncpy(request->hier.cd_host, p->host, sizeof(request->hier.cd_host));
     else
         *request->hier.cd_host = '\0';
 
     request->hier.cd_lookup = lookup;
-
-    debugs(15, 4, "peerNoteDigestLookup: peer " <<
-           (p ? p->host : "<none>") << ", lookup: " <<
-           lookup_t_str[lookup]  );
-
+    debugs(15, 4, "peerNoteDigestLookup: peer " << (p? p->host : "<none>") << ", lookup: " << lookup_t_str[lookup]  );
 #endif
 }
 
@@ -943,7 +961,7 @@ neighborCountIgnored(peer * p)
 static peer *non_peers = NULL;
 
 static void
-neighborIgnoreNonPeer(const IpAddress &from, icp_opcode opcode)
+neighborIgnoreNonPeer(const Ip::Address &from, icp_opcode opcode)
 {
     peer *np;
 
@@ -995,21 +1013,20 @@ ignoreMulticastReply(peer * p, MemObject * mem)
     return 1;
 }
 
-/* I should attach these records to the entry.  We take the first
+/**
+ * I should attach these records to the entry.  We take the first
  * hit we get our wait until everyone misses.  The timeout handler
  * call needs to nip this shopping list or call one of the misses.
  *
  * If a hit process is already started, then sobeit
  */
 void
-
-neighborsUdpAck(const cache_key * key, icp_common_t * header, const IpAddress &from)
+neighborsUdpAck(const cache_key * key, icp_common_t * header, const Ip::Address &from)
 {
     peer *p = NULL;
     StoreEntry *entry;
     MemObject *mem = NULL;
     peer_t ntype = PEER_NONE;
-    char *opcode_d;
     icp_opcode opcode = (icp_opcode) header->opcode;
 
     debugs(15, 6, "neighborsUdpAck: opcode " << opcode << " '" << storeKeyText(key) << "'");
@@ -1023,7 +1040,7 @@ neighborsUdpAck(const cache_key * key, icp_common_t * header, const IpAddress &f
     if (opcode > ICP_END)
         return;
 
-    opcode_d = icp_opcode_str[opcode];
+    const char *opcode_d = icp_opcode_str[opcode];
 
     if (p)
         neighborUpdateRtt(p, mem);
@@ -1363,7 +1380,7 @@ peerProbeConnect(peer * p)
     if (squid_curtime - p->stats.last_connect_probe == 0)
         return ret;/* don't probe to often */
 
-    IpAddress temp(getOutgoingAddr(NULL,p));
+    Ip::Address temp(getOutgoingAddr(NULL,p));
 
     // if IPv6 is disabled try to force IPv4-only outgoing.
     if (!Ip::EnableIpv6 && !temp.SetIPv4()) {
@@ -1563,7 +1580,7 @@ dump_peer_options(StoreEntry * sentry, peer * p)
     if (p->options.userhash)
         storeAppendPrintf(sentry, " userhash");
 
-    if (p->options.sourcehash)
+    if (p->options.userhash)
         storeAppendPrintf(sentry, " sourcehash");
 
     if (p->options.weighted_roundrobin)
@@ -1584,16 +1601,20 @@ dump_peer_options(StoreEntry * sentry, peer * p)
         storeAppendPrintf(sentry, " closest-only");
 
 #if USE_HTCP
-    if (p->options.htcp)
+    if (p->options.htcp) {
         storeAppendPrintf(sentry, " htcp");
-    if (p->options.htcp_oldsquid)
-        storeAppendPrintf(sentry, " htcp-oldsquid");
-    if (p->options.htcp_no_clr)
-        storeAppendPrintf(sentry, " htcp-no-clr");
-    if (p->options.htcp_no_purge_clr)
-        storeAppendPrintf(sentry, " htcp-no-purge-clr");
-    if (p->options.htcp_only_clr)
-        storeAppendPrintf(sentry, " htcp-only-clr");
+        if (p->options.htcp_oldsquid || p->options.htcp_no_clr || p->options.htcp_no_purge_clr || p->options.htcp_only_clr) {
+            int doneopts=0;
+            if (p->options.htcp_oldsquid)
+                storeAppendPrintf(sentry, "%soldsquid",(doneopts++>0?",":"="));
+            if (p->options.htcp_no_clr)
+                storeAppendPrintf(sentry, "%sno-clr",(doneopts++>0?",":"="));
+            if (p->options.htcp_no_purge_clr)
+                storeAppendPrintf(sentry, "%sno-purge-clr",(doneopts++>0?",":"="));
+            if (p->options.htcp_only_clr)
+                storeAppendPrintf(sentry, "%sonly-clr",(doneopts++>0?",":"="));
+        }
+    }
 #endif
 
     if (p->options.no_netdb_exchange)
@@ -1758,7 +1779,7 @@ dump_peers(StoreEntry * sentry, peer * peers)
 
 #if USE_HTCP
 void
-neighborsHtcpReply(const cache_key * key, htcpReplyData * htcp, const IpAddress &from)
+neighborsHtcpReply(const cache_key * key, htcpReplyData * htcp, const Ip::Address &from)
 {
     StoreEntry *e = Store::Root().get(key);
     MemObject *mem = NULL;
