@@ -35,6 +35,7 @@
 
 #include "squid.h"
 #include "event.h"
+#include "fde.h"
 #include "Store.h"
 #include "CacheManager.h"
 #include "StoreClient.h"
@@ -50,6 +51,7 @@
 #endif
 #include "Stack.h"
 #include "SquidTime.h"
+#include "swap_log_op.h"
 
 static STMCB storeWriteComplete;
 
@@ -57,6 +59,9 @@ static STMCB storeWriteComplete;
 
 #define STORE_IN_MEM_BUCKETS            (229)
 
+
+/** \todo Convert these string constants to enum string-arrays generated */
+
 const char *memStatusStr[] = {
     "NOT_IN_MEMORY",
     "IN_MEMORY"
@@ -99,7 +104,6 @@ static storerepl_entry_t *storerepl_list = NULL;
 /*
  * local function prototypes
  */
-static void storeGetMemSpace(int);
 static int getKeyCounter(void);
 static OBJH storeCheckCachableStats;
 static EVH storeLateRelease;
@@ -165,7 +169,7 @@ StoreEntry::operator new (size_t bytecount)
 void
 StoreEntry::operator delete (void *address)
 {
-    pool->free(address);
+    pool->freeOne(address);
 }
 
 void
@@ -314,11 +318,18 @@ StoreEntry::storeClientType() const
     if (store_status == STORE_OK) {
         /* the object has completed. */
 
-        if (mem_obj->inmem_lo == 0 && !isEmpty())
-            /* hot object */
-            return STORE_MEM_CLIENT;
-        else
-            return STORE_DISK_CLIENT;
+        if (mem_obj->inmem_lo == 0 && !isEmpty()) {
+            if (swap_status == SWAPOUT_DONE) {
+                if (mem_obj->endOffset() == mem_obj->object_sz) {
+                    /* hot object fully swapped in */
+                    return STORE_MEM_CLIENT;
+                }
+            } else {
+                /* Memory-only, or currently being swapped out */
+                return STORE_MEM_CLIENT;
+            }
+        }
+        return STORE_DISK_CLIENT;
     }
 
     /* here and past, entry is STORE_PENDING */
@@ -804,6 +815,7 @@ void
 StoreEntry::write (StoreIOBuffer writeBuffer)
 {
     assert(mem_obj != NULL);
+    assert(writeBuffer.length >= 0);
     /* This assert will change when we teach the store to update */
     PROF_start(StoreEntry_write);
     assert(store_status == STORE_PENDING);
@@ -1110,8 +1122,10 @@ StoreEntry::abort()
     unlock();       /* unlock */
 }
 
-/* Clear Memory storage to accommodate the given object len */
-static void
+/**
+ * Clear Memory storage to accommodate the given object len
+ */
+void
 storeGetMemSpace(int size)
 {
     PROF_start(storeGetMemSpace);
@@ -1404,7 +1418,13 @@ StoreEntry::keepInMemory() const
     if (mem_obj->data_hdr.size() == 0)
         return 0;
 
-    return mem_obj->inmem_lo == 0;
+    if (mem_obj->inmem_lo != 0)
+        return 0;
+
+    if (!Config.onoff.memory_cache_first && swap_status == SWAPOUT_DONE && refcount == 1)
+        return 0;
+
+    return 1;
 }
 
 int
@@ -1431,10 +1451,8 @@ StoreEntry::checkNegativeHit() const
 void
 StoreEntry::negativeCache()
 {
-    // XXX: should make the default for expires 0 instead of -1
-    //      so we can distinguish "Expires: -1" from nothing.
-    if (expires <= 0)
-#if HTTP_VIOLATIONS
+    if (expires == 0)
+#if USE_HTTP_VIOLATIONS
         expires = squid_curtime + Config.negativeTtl;
 #else
         expires = squid_curtime;
@@ -1487,20 +1505,19 @@ StoreEntry::timestampsSet()
     const HttpReply *reply = getReply();
     time_t served_date = reply->date;
     int age = reply->header.getInt(HDR_AGE);
-    /* Compute the timestamp, mimicking RFC2616 section 13.2.3. */
+    /*
+     * The timestamp calculations below tries to mimic the properties
+     * of the age calculation in RFC2616 section 13.2.3. The implementaion
+     * isn't complete, and the most notable exception from the RFC is that
+     * this does not account for response_delay, but it probably does
+     * not matter much as this is calculated immediately when the headers
+     * are received, not when the whole response has been received.
+     */
     /* make sure that 0 <= served_date <= squid_curtime */
 
     if (served_date < 0 || served_date > squid_curtime)
         served_date = squid_curtime;
 
-    /* Bug 1791:
-     * If the returned Date: is more than 24 hours older than
-     * the squid_curtime, then one of us needs to use NTP to set our
-     * clock.  We'll pretend that our clock is right.
-     */
-    else if (served_date < (squid_curtime - 24 * 60 * 60) )
-        served_date = squid_curtime;
-
     /*
      * Compensate with Age header if origin server clock is ahead
      * of us and there is a cache in between us and the origin
@@ -1511,14 +1528,6 @@ StoreEntry::timestampsSet()
         if (squid_curtime > age)
             served_date = squid_curtime - age;
 
-    // compensate for Squid-to-server and server-to-Squid delays
-    if (mem_obj && mem_obj->request) {
-        const time_t request_sent =
-            mem_obj->request->hier.peer_http_request_sent.tv_sec;
-        if (0 < request_sent && request_sent < squid_curtime)
-            served_date -= (squid_curtime - request_sent);
-    }
-
     if (reply->expires > 0 && reply->date > -1)
         expires = served_date + (reply->expires - reply->date);
     else
@@ -1698,10 +1707,13 @@ void
 storeReplAdd(const char *type, REMOVALPOLICYCREATE * create)
 {
     int i;
-    /* find the number of currently known repl types */
 
+    /* find the number of currently known repl types */
     for (i = 0; storerepl_list && storerepl_list[i].typestr; i++) {
-        assert(strcmp(storerepl_list[i].typestr, type) != 0);
+        if (strcmp(storerepl_list[i].typestr, type) == 0) {
+            debugs(20, 1, "WARNING: Trying to load store replacement policy " << type << " twice.");
+            return;
+        }
     }
 
     /* add the new type */
@@ -1834,11 +1846,11 @@ StoreEntry::trimMemory()
     if (mem_status == IN_MEMORY)
         return;
 
-    if (mem_obj->policyLowestOffsetToKeep() == 0)
-        /* Nothing to do */
-        return;
-
     if (!swapOutAble()) {
+        if (mem_obj->policyLowestOffsetToKeep(0) == 0) {
+            /* Nothing to do */
+            return;
+        }
         /*
          * Its not swap-able, and we're about to delete a chunk,
          * so we must make it PRIVATE.  This is tricky/ugly because
@@ -1891,51 +1903,6 @@ StoreEntry::modifiedSince(HttpRequest * request) const
     }
 }
 
-bool
-StoreEntry::hasIfMatchEtag(const HttpRequest &request) const
-{
-    const String reqETags = request.header.getList(HDR_IF_MATCH);
-    return hasOneOfEtags(reqETags, false);
-}
-
-bool
-StoreEntry::hasIfNoneMatchEtag(const HttpRequest &request) const
-{
-    const String reqETags = request.header.getList(HDR_IF_NONE_MATCH);
-    // weak comparison is allowed only for HEAD or full-body GET requests
-    const bool allowWeakMatch = !request.flags.range &&
-                                (request.method == METHOD_GET || request.method == METHOD_HEAD);
-    return hasOneOfEtags(reqETags, allowWeakMatch);
-}
-
-/// whether at least one of the request ETags matches entity ETag
-bool
-StoreEntry::hasOneOfEtags(const String &reqETags, const bool allowWeakMatch) const
-{
-    const ETag repETag = getReply()->header.getETag(HDR_ETAG);
-    if (!repETag.str)
-        return strListIsMember(&reqETags, "*", ',');
-
-    bool matched = false;
-    const char *pos = NULL;
-    const char *item;
-    int ilen;
-    while (!matched && strListGetItem(&reqETags, ',', &item, &ilen, &pos)) {
-        if (!strncmp(item, "*", ilen))
-            matched = true;
-        else {
-            String str;
-            str.append(item, ilen);
-            ETag reqETag;
-            if (etagParseInit(&reqETag, str.termedBuf())) {
-                matched = allowWeakMatch ? etagIsWeakEqual(repETag, reqETag) :
-                          etagIsStrongEqual(repETag, reqETag);
-            }
-        }
-    }
-    return matched;
-}
-
 StorePointer
 StoreEntry::store() const
 {
@@ -1993,6 +1960,6 @@ NullStoreEntry::getSerialisedMetaData()
     return NULL;
 }
 
-#ifndef _USE_INLINE_
+#if !_USE_INLINE_
 #include "Store.cci"
 #endif
