@@ -1,3 +1,4 @@
+#define SQUID_UNIT_TEST 1
 #include "config.h"
 #include <cppunit/TestAssert.h>
 
@@ -5,15 +6,25 @@
 #include "HttpReply.h"
 #include "Mem.h"
 
-/* to avoid libsquid.la and its comm stuff */
-#include "TextException.cc"
-
 CPPUNIT_TEST_SUITE_REGISTRATION( testHttpReply );
 
 struct SquidConfig Config;
 
 /* stub functions to link successfully */
 
+#include "Store.h"
+void
+StoreEntry::timestampsSet()
+{
+    fatal("StoreEntry::timestampsSet. Not implemented.");
+}
+
+void
+StoreEntry::setPublicKey()
+{
+    fatal("StoreEntry::setPulicKey. Not implemented.");
+}
+
 #include "MemObject.h"
 int64_t
 MemObject::endOffset() const
