@@ -64,7 +64,7 @@ public:
     bool noConnections() const;
     char *url;
     char *host;			/* either request->host or proxy host */
-    unsigned short port;
+    u_short port;
     HttpRequest *request;
     FwdServer *servers;
 
@@ -175,7 +175,7 @@ tunnelStateFree(TunnelStateData * tunnelState)
     assert(tunnelState->noConnections());
     safe_free(tunnelState->url);
     FwdState::serversFree(&tunnelState->servers);
-    safe_free(tunnelState->host);
+    tunnelState->host = NULL;
     HTTPMSGUNLOCK(tunnelState->request);
     delete tunnelState;
 }
@@ -308,16 +308,11 @@ TunnelStateData::copy (size_t len, comm_err_t errcode, int xerrno, Connection &f
      */
     cbdataInternalLock(this);	/* ??? should be locked by the caller... */
 
-    /* Bump the source connection timeout on any activity */
-    if (!fd_closed(from.fd()))
-        commSetTimeout(from.fd(), Config.Timeout.read, tunnelTimeout, this);
+    /* Bump the server connection timeout on any activity */
+    if (!fd_closed(server.fd()))
+        commSetTimeout(server.fd(), Config.Timeout.read, tunnelTimeout, this);
 
-    /* Bump the dest connection read timeout on any activity */
-    /* see Bug 3659: tunnels can be weird, with very long one-way transfers */
-    if (!fd_closed(to.fd()))
-        commSetTimeout(to.fd(), Config.Timeout.read, tunnelTimeout, this);
-
-    if (errcode)
+    if (len < 0 || errcode)
         from.error (xerrno);
     else if (len == 0 || fd_closed(to.fd())) {
         comm_close(from.fd());
@@ -348,10 +343,12 @@ TunnelStateData::writeServerDone(char *buf, size_t len, comm_err_t flag, int xer
 {
     debugs(26, 3, "tunnelWriteServer: FD " << server.fd() << ", " << len << " bytes written");
 
+    if (flag == COMM_ERR_CLOSING)
+        return;
+
     /* Error? */
-    if (flag != COMM_OK) {
-        if (flag != COMM_ERR_CLOSING)
-            server.error(xerrno); // may call comm_close
+    if (len < 0 || flag != COMM_OK) {
+        server.error(xerrno); // may call comm_close
         return;
     }
 
@@ -407,10 +404,12 @@ TunnelStateData::writeClientDone(char *buf, size_t len, comm_err_t flag, int xer
 {
     debugs(26, 3, "tunnelWriteClient: FD " << client.fd() << ", " << len << " bytes written");
 
+    if (flag == COMM_ERR_CLOSING)
+        return;
+
     /* Error? */
-    if (flag != COMM_OK) {
-        if (flag != COMM_ERR_CLOSING)
-            client.error(xerrno); // may call comm_close
+    if (len < 0 || flag != COMM_OK) {
+        client.error(xerrno); // may call comm_close
         return;
     }
 
@@ -475,7 +474,7 @@ tunnelConnectTimeout(int fd, void *data)
     if (tunnelState->servers) {
         if (tunnelState->servers->_peer)
             hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
-                          tunnelState->servers->_peer->name);
+                          tunnelState->servers->_peer->host);
         else if (Config.onoff.log_ip_on_direct)
             hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
                           fd_table[tunnelState->server.fd()].ipaddr);
@@ -547,7 +546,7 @@ tunnelErrorComplete(int fdnotused, void *data, size_t sizenotused)
     if (!fd_closed(tunnelState->client.fd()))
         comm_close(tunnelState->client.fd());
 
-    if (!fd_closed(tunnelState->server.fd()))
+    if (fd_closed(tunnelState->server.fd()))
         comm_close(tunnelState->server.fd());
 
     cbdataInternalUnlock(tunnelState);
@@ -565,7 +564,7 @@ tunnelConnectDone(int fdnotused, const DnsLookupDetails &dns, comm_err_t status,
 
     if (tunnelState->servers->_peer)
         hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
-                      tunnelState->servers->_peer->name);
+                      tunnelState->servers->_peer->host);
     else if (Config.onoff.log_ip_on_direct)
         hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
                       fd_table[tunnelState->server.fd()].ipaddr);
@@ -590,7 +589,7 @@ tunnelConnectDone(int fdnotused, const DnsLookupDetails &dns, comm_err_t status,
         err->callback_data = tunnelState;
         errorSend(tunnelState->client.fd(), err);
     } else {
-        if (tunnelState->servers->_peer && !tunnelState->servers->_peer->options.originserver)
+        if (tunnelState->servers->_peer)
             tunnelProxyConnected(tunnelState->server.fd(), tunnelState);
         else {
             tunnelConnected(tunnelState->server.fd(), tunnelState);
@@ -642,7 +641,7 @@ tunnelStart(ClientHttpRequest * http, int64_t * size_ptr, int *status_ptr)
     statCounter.server.all.requests++;
     statCounter.server.other.requests++;
     /* Create socket. */
-    IpAddress temp = getOutgoingAddr(request,NULL);
+    Ip::Address temp = getOutgoingAddr(request,NULL);
 
     // if IPv6 is disabled try to force IPv4-only outgoing.
     if (!Ip::EnableIpv6 && !temp.SetIPv4()) {
@@ -681,9 +680,7 @@ tunnelStart(ClientHttpRequest * http, int64_t * size_ptr, int *status_ptr)
         return;
     }
 
-    // record local IP:port for %<la and %<lp logging
-    if (comm_local_port(sock))
-        request->hier.peer_local_addr = fd_table[sock].local_addr;
+    request->hier.peer_local_port = comm_local_port(sock); // for %<lp logging
 
     tunnelState = new TunnelStateData;
 #if DELAY_POOLS
@@ -763,7 +760,8 @@ tunnelPeerSelectComplete(FwdServer * fs, void *data)
     }
 
     tunnelState->servers = fs;
-    tunnelState->host = fs->_peer ? xstrdup(fs->_peer->host) : xstrdup(request->GetHost());
+    tunnelState->host = fs->_peer ? fs->_peer->host : xstrdup(request->GetHost());
+    request->peer_host = fs->_peer ? fs->_peer->host : NULL;
 
     if (fs->_peer == NULL) {
         tunnelState->port = request->port;
@@ -777,7 +775,7 @@ tunnelPeerSelectComplete(FwdServer * fs, void *data)
 
     if (fs->_peer) {
         tunnelState->request->peer_login = fs->_peer->login;
-        tunnelState->request->flags.proxying = (fs->_peer->options.originserver?0:1);
+        tunnelState->request->flags.proxying = 1;
     } else {
         tunnelState->request->peer_login = NULL;
         tunnelState->request->flags.proxying = 0;
