@@ -45,7 +45,7 @@ static HttpRequest *urlParseFinish(const HttpRequestMethod& method,
                                    const char *const login,
                                    const int port,
                                    HttpRequest *request);
-static HttpRequest *urnParse(const HttpRequestMethod& method, char *urn, HttpRequest *request);
+static HttpRequest *urnParse(const HttpRequestMethod& method, char *urn);
 static const char valid_hostname_chars_u[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "abcdefghijklmnopqrstuvwxyz"
@@ -124,12 +124,6 @@ urlParseProtocol(const char *b, const char *e)
     if (strncasecmp(b, "file", len) == 0)
         return PROTO_FTP;
 
-    if (strncasecmp(b, "coap", len) == 0)
-        return PROTO_COAP;
-
-    if (strncasecmp(b, "coaps", len) == 0)
-        return PROTO_COAPS;
-
     if (strncasecmp(b, "gopher", len) == 0)
         return PROTO_GOPHER;
 
@@ -165,12 +159,6 @@ urlDefaultPort(protocol_t p)
     case PROTO_FTP:
         return 21;
 
-    case PROTO_COAP:
-    case PROTO_COAPS:
-        // coaps:// default is TBA as of draft-ietf-core-coap-08.
-        // Assuming IANA policy of allocating same port for base and TLS protocol versions will occur.
-        return 5683;
-
     case PROTO_GOPHER:
         return 70;
 
@@ -247,7 +235,7 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
         port = urlDefaultPort(protocol);
         return urlParseFinish(method, protocol, url, host, login, port, request);
     } else if (!strncmp(url, "urn:", 4)) {
-        return urnParse(method, url, request);
+        return urnParse(method, url);
     } else {
         /* Parse the URL: */
         src = url;
@@ -261,16 +249,15 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
         *dst = '\0';
 
         /* Then its :// */
-        if ((i+3) > l || *src != ':' || *(src + 1) != '/' || *(src + 2) != '/')
+        /* (XXX yah, I'm not checking we've got enough data left before checking the array..) */
+        if (*src != ':' || *(src + 1) != '/' || *(src + 2) != '/')
             return NULL;
         i += 3;
         src += 3;
 
         /* Then everything until first /; thats host (and port; which we'll look for here later) */
-        // bug 1881: If we don't get a "/" then we imply it was there
-        // bug 3074: We could just be given a "?" or "#". These also imply "/"
-        // bug 3233: whitespace is also a hostname delimiter.
-        for (dst = host; i < l && *src != '/' && *src != '?' && *src != '#' && *src != '\0' && !xisspace(*src); i++, src++, dst++) {
+        /* bug 1881: If we don't get a "/" then we imply it was there */
+        for (dst = host; i < l && *src != '/' && *src != '\0'; i++, src++, dst++) {
             *dst = *src;
         }
 
@@ -283,15 +270,8 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
             return NULL;
         *dst = '\0';
 
-        // bug 3074: received 'path' starting with '?', '#', or '\0' implies '/'
-        if (*src == '?' || *src == '#' || *src == '\0') {
-            urlpath[0] = '/';
-            dst = &urlpath[1];
-        } else {
-            dst = urlpath;
-        }
         /* Then everything from / (inclusive) until \r\n or \0 - thats urlpath */
-        for (; i < l && *src != '\r' && *src != '\n' && *src != '\0'; i++, src++, dst++) {
+        for (dst = urlpath; i < l && *src != '\r' && *src != '\n' && *src != '\0'; i++, src++, dst++) {
             *dst = *src;
         }
 
@@ -345,12 +325,6 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
             }
         }
 
-        // Bug 3183 sanity check: If scheme is present, host must be too.
-        if (protocol != PROTO_NONE && (host == NULL || *host == '\0')) {
-            debugs(23, DBG_IMPORTANT, "SECURITY ALERT: Missing hostname in URL '" << url << "'. see access.log for details.");
-            return NULL;
-        }
-
         if (t && *t == ':') {
             *t = '\0';
             t++;
@@ -399,7 +373,7 @@ urlParse(const HttpRequestMethod& method, char *url, HttpRequest *request)
         return NULL;
     }
 
-#ifdef HARDCODE_DENY_PORTS
+#if HARDCODE_DENY_PORTS
     /* These ports are filtered in the default squid.conf, but
      * maybe someone wants them hardcoded... */
     if (port == 7 || port == 9 || port == 19) {
@@ -461,26 +435,18 @@ urlParseFinish(const HttpRequestMethod& method,
         request = new HttpRequest(method, protocol, urlpath);
     else {
         request->initHTTP(method, protocol, urlpath);
-        safe_free(request->canonical);
     }
 
     request->SetHost(host);
     xstrncpy(request->login, login, MAX_LOGIN_SZ);
-    request->port = (unsigned short) port;
+    request->port = (u_short) port;
     return request;
 }
 
 static HttpRequest *
-urnParse(const HttpRequestMethod& method, char *urn, HttpRequest *request)
+urnParse(const HttpRequestMethod& method, char *urn)
 {
     debugs(50, 5, "urnParse: " << urn);
-
-    if (request) {
-        request->initHTTP(method, PROTO_URN, urn + 4);
-        safe_free(request->canonical);
-        return request;
-    }
-
     return new HttpRequest(method, PROTO_URN, urn + 4);
 }
 
@@ -833,7 +799,7 @@ urlCheckRequest(const HttpRequest * r)
     // we support OPTIONS and TRACE directed at us (with a 501 reply, for now)
     // we also support forwarding OPTIONS and TRACE, except for the *-URI ones
     if (r->method == METHOD_OPTIONS || r->method == METHOD_TRACE)
-        return (r->header.getInt64(HDR_MAX_FORWARDS) == 0 || r->urlpath != "*");
+        return (r->max_forwards == 0 || r->urlpath != "*");
 
     if (r->method == METHOD_PURGE)
         return 1;
@@ -867,7 +833,7 @@ urlCheckRequest(const HttpRequest * r)
         break;
 
     case PROTO_HTTPS:
-#ifdef USE_SSL
+#if USE_SSL
 
         rc = 1;
 
